@@ -1,3 +1,4 @@
+const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
 const GLib = imports.gi.GLib;
@@ -14,6 +15,7 @@ const {SignalManager} = imports.misc.signalManager;
 const {each, find, findIndex, filter, throttle, unref, trySpawnCommandLine} = imports.misc.util;
 const {createStore} = imports.misc.state;
 
+const AppGroup = require('./appGroup');
 const AppList = require('./appList');
 const {
   RESERVE_KEYS,
@@ -99,9 +101,7 @@ class PinnedFavs {
         if (!opts.app) {
             return false;
         }
-        if (!opts.pos) {
-            opts.pos = -1;
-        }
+
         let newFavorite = {
             id: opts.appId,
             app: opts.app
@@ -159,11 +159,7 @@ class PinnedFavs {
                 return favorite.id === opts.appId;
             });
         }
-        let newIndex = opts.pos;
-        if (oldIndex > -1 && newIndex > oldIndex) {
-            newIndex = newIndex - 1;
-        }
-        this._favorites.splice(newIndex, 0, this._favorites.splice(oldIndex, 1)[0]);
+        this._favorites.splice(opts.pos, 0, this._favorites.splice(oldIndex, 1)[0]);
         this._favorites = filter(this._favorites, function(favorite) {
             return favorite && favorite.app != null;
         });
@@ -199,14 +195,18 @@ class GroupedWindowListApplet extends Applet.Applet {
             currentWs: global.screen.get_active_workspace_index(),
             panelEditMode: global.settings.get_boolean('panel-edit-mode'),
             menuOpen: false,
-            dragPlaceholder: null,
-            dragPlaceholderPos: -1,
-            animatingPlaceholdersCount: 0,
+            dragging: {
+                posList: null,
+                dragPlaceholder: null,
+                pos: -1,
+                isForeign: null,
+            },
+            appletActor: null,
             appletReady: false,
             willUnmount: false,
             settings: {},
             homeDir: GLib.get_home_dir(),
-            overlayPreview: null,
+            lastOverlayPreview: null,
             lastCycled: -1,
             lastTitleDisplay: null,
             scrollActive: false,
@@ -227,7 +227,7 @@ class GroupedWindowListApplet extends Applet.Applet {
             getPanelIconSize: () => this.getPanelIconSize(),
             getPanelMonitor: () => this.panel ? Main.layoutManager.monitors[this.panel.monitorIndex] : null,
             getAppSystem: () => Cinnamon.AppSystem.get_default(),
-            getAppFromWMClass: (specialApps, metaWindow) => this.getAppFromWMClass(specialApps, metaWindow),
+            getAppFromWindow: (metaWindow) => this.getAppFromWindow(metaWindow),
             getTracker: () => this.tracker,
             addWindowToAllWorkspaces: (win, app, isFavoriteApp) => {
                 each(this.appLists, function(appList) {
@@ -253,7 +253,7 @@ class GroupedWindowListApplet extends Applet.Applet {
             refreshCurrentAppList: () => this.refreshCurrentAppList(),
             refreshAllAppLists: () => this.refreshAllAppLists(),
             getCurrentAppList: () => this.getCurrentAppList(),
-            clearDragPlaceholder: () => this.clearDragPlaceholder(),
+            moveLauncher: (source) => this.moveLauncher(source),
             getAutoStartApps: () => this.getAutoStartApps(),
             getRecentItems: () =>
                 Gtk.RecentManager.get_default()
@@ -268,6 +268,7 @@ class GroupedWindowListApplet extends Applet.Applet {
             cycleWindows: (e, source) => this.handleScroll(e, source),
             openAbout: () => this.openAbout(),
             configureApplet: () => this.configureApplet(),
+            removeApplet: (event) => this.confirmRemoveApplet(event),
         });
 
         this.settings = new AppletSettings(this.state.settings, metadata.uuid, instance_id);
@@ -286,14 +287,18 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.setAllowedLayout(Applet.AllowedLayout.BOTH);
         Gettext.bindtextdomain(metadata.uuid, GLib.get_home_dir() + '/.local/share/locale');
 
+        this.actor.set_style_class_name('grouped-window-list-box');
+        this.state.set({appletActor: this.actor});
+        this.on_orientation_changed(orientation);
+
         this.getAutoStartApps();
-        this.onSwitchWorkspace = throttle(this.onSwitchWorkspace, 100, true);
+        this.onSwitchWorkspace = throttle(this.onSwitchWorkspace, 35, false); //Note: causes a 35ms delay in execution
         this.signals.connect(this.actor, 'scroll-event', (c, e) => this.handleScroll(e));
         this.signals.connect(global, 'scale-changed', (...args) => this.onUIScaleChange(...args));
         this.signals.connect(global.window_manager, 'switch-workspace', (...args) => this.onSwitchWorkspace(...args));
         this.signals.connect(global.screen, 'workspace-removed', (...args) => this.onWorkspaceRemoved(...args));
         this.signals.connect(global.screen, 'window-monitor-changed', (...args) => this.onWindowMonitorChanged(...args));
-        this.signals.connect(global.screen, 'monitors-changed', (...args) => this.on_applet_instances_changed(...args));
+        this.signals.connect(Main.panelManager, 'monitors-changed', (...args) => this.on_applet_instances_changed(...args));
         this.signals.connect(global.screen, 'window-skip-taskbar-changed', (...args) => this.onWindowSkipTaskbarChanged(...args));
         this.signals.connect(global.display, 'window-marked-urgent', (...args) => this.updateAttentionState(...args));
         this.signals.connect(global.display, 'window-demands-attention', (...args) => this.updateAttentionState(...args));
@@ -309,11 +314,15 @@ class GroupedWindowListApplet extends Applet.Applet {
     bindSettings() {
         let settingsProps = [
             {key: 'group-apps', value: 'groupApps', cb: this.refreshCurrentAppList},
-            {key: 'enable-app-button-dragging', value: 'enableDragging', cb: null},
+            {key: 'enable-app-button-dragging', value: 'enableDragging', cb: this.draggableSettingChanged},
             {key: 'launcher-animation-effect', value: 'launcherAnimationEffect', cb: null},
             {key: 'pinned-apps', value: 'pinnedApps', cb: null},
             {key: 'middle-click-action', value: 'middleClickAction', cb: null},
             {key: 'left-click-action', value: 'leftClickAction', cb: null},
+            {key: 'show-apps-order-hotkey', value: 'showAppsOrderHotkey', cb: this.bindAppKeys},
+            {key: 'show-apps-order-timeout', value: 'showAppsOrderTimeout', cb: null},
+            {key: 'super-num-hotkeys', value: 'SuperNumHotkeys', cb: this.bindAppKeys},
+            {key: 'cycleMenusHotkey', value: 'cycleMenusHotkey', cb: this.bindAppKeys},
             {key: 'enable-hover-peek', value: 'enablePeek', cb: null},
             {key: 'onclick-thumbnails', value: 'onClickThumbs', cb: null},
             {key: 'hover-peek-opacity', value: 'peekOpacity', cb: null},
@@ -336,7 +345,6 @@ class GroupedWindowListApplet extends Applet.Applet {
             {key: 'scroll-behavior', value: 'scrollBehavior', cb: null},
             {key: 'show-recent', value: 'showRecent', cb: null},
             {key: 'autostart-menu-item', value: 'autoStart', cb: null},
-            {key: 'launch-new-instance-menu-item', value: 'launchNewInstance', cb: null},
             {key: 'monitor-move-all-windows', value: 'monitorMoveAllWindows', cb: null},
             {key: 'show-all-workspaces', value: 'showAllWorkspaces', cb: this.refreshAllAppLists}
         ];
@@ -352,10 +360,19 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.state.set({lastTitleDisplay: this.state.settings.titleDisplay});
     }
 
+    draggableSettingChanged() {
+        each(this.appLists, (workspace) => {
+            each(workspace.appList, (appGroup) => {
+                appGroup._draggable.inhibit = !this.state.settings.enableDragging;
+            });
+        });
+    }
+
     on_applet_added_to_panel() {
         if (this.state.appletReady && this.state.panelEditMode) {
             return;
         }
+        this.bindAppKeys();
         this.state.set({appletReady: true});
     }
 
@@ -394,6 +411,7 @@ class GroupedWindowListApplet extends Applet.Applet {
             orientation: orientation,
             isHorizontal: orientation === St.Side.TOP || orientation === St.Side.BOTTOM
         });
+
         if (this.state.isHorizontal) {
             this.actor.remove_style_class_name('vertical');
         } else {
@@ -404,6 +422,7 @@ class GroupedWindowListApplet extends Applet.Applet {
 
     on_applet_removed_from_panel() {
         this.state.set({willUnmount: true});
+        this.unbindAppKeys();
         this.signals.disconnectAllSignals();
         for (let i = 0, len = this.appLists.length; i < len; i++) {
             if (this.appLists[i]) {
@@ -434,8 +453,60 @@ class GroupedWindowListApplet extends Applet.Applet {
         }
     }
 
+    bindAppKeys() {
+        this.unbindAppKeys();
+
+        for (let i = 1; i < 10; i++) {
+            if (this.state.settings.SuperNumHotkeys) {
+                this.bindAppKey(i);
+            }
+            this.bindNewAppKey(i);
+        }
+        Main.keybindingManager.addHotKey('launch-show-apps-order', this.state.settings.showAppsOrderHotkey, () =>
+            this.showAppsOrder()
+        );
+        Main.keybindingManager.addHotKey('launch-cycle-menus', this.state.settings.cycleMenusHotkey, () =>
+            this.cycleMenus()
+        );
+    }
+
+    unbindAppKeys() {
+        for (let i = 1; i < 10; i++) {
+            Main.keybindingManager.removeHotKey('launch-app-key-' + i);
+            Main.keybindingManager.removeHotKey('launch-new-app-key-' + i);
+        }
+        Main.keybindingManager.removeHotKey('launch-show-apps-order');
+        Main.keybindingManager.removeHotKey('launch-cycle-menus');
+    }
+
+    bindAppKey(i) {
+        Main.keybindingManager.addHotKey('launch-app-key-' + i, '<Super>' + i, () => this.onAppKeyPress(i));
+    }
+
+    bindNewAppKey(i) {
+        Main.keybindingManager.addHotKey('launch-new-app-key-' + i, '<Super><Shift>' + i, () =>
+            this.onNewAppKeyPress(i)
+        );
+    }
+
+    onAppKeyPress(number) {
+        this.getCurrentAppList().onAppKeyPress(number);
+    }
+
+    onNewAppKeyPress(number) {
+        this.getCurrentAppList().onNewAppKeyPress(number);
+    }
+
+    showAppsOrder() {
+        this.getCurrentAppList().showAppsOrder();
+    }
+
+    cycleMenus() {
+        this.getCurrentAppList().cycleMenus();
+    }
+
     handleMonitorWindowsPrefsChange(value) {
-        
+
     }
 
     updateMonitorWatchlist() {
@@ -544,7 +615,7 @@ class GroupedWindowListApplet extends Applet.Applet {
         each(this.appLists, (workspace) => {
             each(workspace.appList, (appGroup) => {
                 if (titleDisplay === TitleDisplay.Focused) {
-                    appGroup.hideLabel(false);
+                    appGroup.hideLabel();
                 }
                 appGroup.handleTitleDisplayChange();
             });
@@ -553,20 +624,19 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.state.set({lastTitleDisplay: titleDisplay});
     }
 
-    getAppFromWMClass(specialApps, metaWindow) {
-        let startupClass = (wmClass) => {
-            let app = null;
-            for (let i = 0, len = specialApps.length; i < len; i++) {
-                if (specialApps[i].wmClass === wmClass) {
-                    app = this.appSystem.lookup_app(specialApps[i].id);
-                    if (app) {
-                        app.wmClass = wmClass;
-                    }
-                }
-            }
-            return app;
-        };
-        return startupClass(metaWindow.get_wm_class_instance());
+    getAppFromWindow(metaWindow) {
+        let tracker = this.state.trigger('getTracker');
+        if (!tracker) {
+          return null;
+        }
+        let app = tracker.get_window_app(metaWindow);
+        if (!app) {
+          app = tracker.get_app_from_pid(metaWindow.get_pid());
+        }
+        if (!app) {
+          app = tracker.get_app_from_pid(metaWindow.get_client_pid());
+        }
+        return app;
     }
 
     getCurrentAppList() {
@@ -609,186 +679,214 @@ class GroupedWindowListApplet extends Applet.Applet {
     }
 
     handleScroll(e, sourceFromAppGroup) {
-        if ((this.state.settings.scrollBehavior === 1 && this.state.settings.leftClickAction !== 3)
-            || (e && sourceFromAppGroup && !this.state.settings.thumbnailScrollBehavior)) {
+        if (e?.get_scroll_direction() == Clutter.ScrollDirection.SMOOTH)
             return;
-        }
 
-        this.state.set({scrollActive: true});
+        if( (this.state.settings.thumbnailScrollBehavior) || (this.state.settings.scrollBehavior === 2) ||
+            (this.state.settings.leftClickAction === 3 && this.state.settings.scrollBehavior !== 3
+             && !e && sourceFromAppGroup)  ||
+            (this.state.settings.leftClickAction !== 3 && this.state.settings.scrollBehavior === 3               
+                        && e && !sourceFromAppGroup)  || 
+            (this.state.settings.leftClickAction === 3 && this.state.settings.scrollBehavior === 3)) {
 
-        let isAppScroll = this.state.settings.scrollBehavior === 2;
-        let direction, source;
+            this.state.set({scrollActive: true});
 
-        if (sourceFromAppGroup) {
-            isAppScroll = false;
-            direction = e ? e.get_scroll_direction() : 1;
-            source = sourceFromAppGroup;
-        } else {
-            direction = e.get_scroll_direction();
-            source = e.get_source()._delegate;
-        }
-        let lastFocusedApp, z, count
+            let isAppScroll = this.state.settings.scrollBehavior === 2;
+            let direction, source;
 
-        if (isAppScroll) {
-            lastFocusedApp = this.appLists[this.state.currentWs].listState.lastFocusedApp;
-            if (!lastFocusedApp) {
-                lastFocusedApp = this.appLists[this.state.currentWs].appList[0].groupState.appId
-            }
-            let focusedIndex = findIndex(this.appLists[this.state.currentWs].appList, function(appGroup) {
-                return appGroup.groupState.metaWindows.length > 0 && appGroup.groupState.appId === lastFocusedApp;
-            });
-            z = direction === 0 ? focusedIndex - 1 : focusedIndex + 1;
-            count = this.appLists[this.state.currentWs].appList.length - 1;
-        } else {
-            if (!source.groupState || source.groupState.metaWindows.length < 1) {
-                return;
-            }
-            let focusedIndex = findIndex(source.groupState.metaWindows, function(metaWindow) {
-                return metaWindow === source.groupState.lastFocused;
-            });
-            z = direction === 0 ? focusedIndex - 1 : focusedIndex + 1;
-            count = source.groupState.metaWindows.length - 1;
-        }
-
-        let limit = count * 2;
-
-        while ((isAppScroll
-            && (!this.appLists[this.state.currentWs].appList[z]
-                || !this.appLists[this.state.currentWs].appList[z].groupState.lastFocused))
-            || (!isAppScroll &&
-                (!source.groupState.metaWindows[z]
-                    || source.groupState.metaWindows[z] === source.groupState.lastFocused))) {
-            limit--;
-            if (direction === 0) {
-                z -= 1;
+            if (sourceFromAppGroup) {
+                isAppScroll = false;
+                direction = e ? e.get_scroll_direction() : 1;
+                source = sourceFromAppGroup;
             } else {
-                z += 1;
+                direction = e.get_scroll_direction();
+                source = e.get_source()._delegate;
             }
-            if (limit < 0) {
-                if (count === 0) {
+            let lastFocusedApp, z, count
+
+            if (isAppScroll) {
+                lastFocusedApp = this.appLists[this.state.currentWs].listState.lastFocusedApp;
+                if (!lastFocusedApp) {
+                    lastFocusedApp = this.appLists[this.state.currentWs].appList[0].groupState.appId
+                }
+                let focusedIndex = findIndex(this.appLists[this.state.currentWs].appList, function(appGroup) {
+                    return appGroup.groupState.metaWindows.length > 0 && appGroup.groupState.appId === lastFocusedApp;
+                });
+                z = direction === 0 ? focusedIndex - 1 : focusedIndex + 1;
+                count = this.appLists[this.state.currentWs].appList.length - 1;
+            } else {
+                if (!source.groupState || source.groupState.metaWindows.length < 1) {
+                    return;
+                }
+                let focusedIndex = findIndex(source.groupState.metaWindows, function(metaWindow) {
+                    return metaWindow === source.groupState.lastFocused;
+                });
+                z = direction === 0 ? focusedIndex - 1 : focusedIndex + 1;
+                count = source.groupState.metaWindows.length - 1;
+            }
+
+            let limit = count * 2;
+
+            while ((isAppScroll
+                && (!this.appLists[this.state.currentWs].appList[z]
+                    || !this.appLists[this.state.currentWs].appList[z].groupState.lastFocused))
+                || (!isAppScroll &&
+                    (!source.groupState.metaWindows[z]
+                        || source.groupState.metaWindows[z] === source.groupState.lastFocused))) {
+                limit--;
+                if (direction === 0) {
+                    z -= 1;
+                } else {
+                    z += 1;
+                }
+                if (limit < 0) {
+                    if (count === 0) {
+                        z = 0;
+                    }
+                    break;
+                } else if (z < 0) {
+                    z = count;
+                } else if (z > count) {
                     z = 0;
                 }
-                break;
-            } else if (z < 0) {
-                z = count;
-            } else if (z > count) {
-                z = 0;
             }
-        }
 
-        let _window = isAppScroll ?
-            this.appLists[this.state.currentWs].appList[z].groupState.lastFocused
-            : source.groupState.metaWindows[z];
-        Main.activateWindow(_window, global.get_current_time());
-        setTimeout(() => this.state.set({scrollActive: false}, 4000));
+            let _window = isAppScroll ?
+                this.appLists[this.state.currentWs].appList[z].groupState.lastFocused
+                : source.groupState.metaWindows[z];
+            Main.activateWindow(_window, global.get_current_time());
+            setTimeout(() => this.state.set({scrollActive: false}, 4000));
+        }
     }
 
     handleDragOver(source, actor, x, y) {
-        if (!this.state.settings.enableDragging || this.state.panelEditMode) {
+        if (!this.state.settings.enableDragging || this.state.panelEditMode)
             return DND.DragMotionResult.NO_DROP;
-        }
-
-        if (!source.actor) return DND.DragMotionResult.CONTINUE;
+        if(actor.name === 'xdnd-proxy-actor')
+            return DND.DragMotionResult.CONTINUE;
 
         let appList = this.appLists[this.state.currentWs];
-        let children = appList.actor.get_children();
-        let windowPos = children.indexOf(source.actor);
-        let isForeign = typeof source.groupState === 'undefined';
+        let rtl_horizontal = this.state.isHorizontal
+            && St.Widget.get_default_direction () === St.TextDirection.RTL;
 
-        let pos = 0;
+        let axis = this.state.isHorizontal ? [x, 'x2'] : [y, 'y2'];
+        if(rtl_horizontal)
+            axis[0] = this.actor.width - axis[0];
 
-        let isHorizontal = appList.actor.height > appList.actor.width;
-        let axis = isHorizontal ? [y, 'y1'] : [x, 'x1'];
-        each(children, (child, i) => {
-            if (axis[0] > children[i].get_allocation_box()[axis[1]] + children[i].width / 2) {
-                pos = i;
+        // save data on drag start
+        if(this.state.dragging.posList === null){
+            this.state.dragging.isForeign = !(source instanceof AppGroup);
+            let children = appList.actor.get_children();
+            this.state.dragging.posList = [];
+            for(let i = 0; i < children.length; i++){
+                let childPos;
+                if(rtl_horizontal)
+                    childPos = this.actor.width - children[i].get_allocation_box()['x1'];
+                else
+                    childPos = children[i].get_allocation_box()[axis[1]];
+                this.state.dragging.posList.push(childPos);
             }
-        });
-
-        if (pos !== this.state.dragPlaceholderPos) {
-            this.state.dragPlaceholderPos = pos;
-
-            // Don't allow positioning before or after self
-            if (windowPos !== -1 && pos === windowPos) {
-                if (this.state.dragPlaceholder) {
-                    this.state.dragPlaceholder.animateOutAndDestroy();
-                    this.state.animatingPlaceholdersCount++;
-                    this.state.dragPlaceholder.actor.connect('destroy', () => this.state.animatingPlaceholdersCount--);
-                }
-                this.state.dragPlaceholder = null;
-
-                return DND.DragMotionResult.CONTINUE;
-            }
-
-            // If the placeholder already exists, we just move
-            // it, but if we are adding it, expand its size in
-            // an animation
-            let fadeIn;
-            if (this.state.dragPlaceholder) {
-                this.state.dragPlaceholder.actor.destroy();
-                fadeIn = false;
-            } else {
-                fadeIn = true;
-            }
-
-            let iconSize = this.getPanelIconSize();
-            this.state.dragPlaceholder = new DND.GenericDragPlaceholderItem();
-            this.state.dragPlaceholder.child.width = iconSize;
-            this.state.dragPlaceholder.child.height = iconSize;
-
-            // For menu items, don't insert actors at specific indices,
-            // they will get attached to the end of the list anyway.
-            if (!isForeign) {
-                appList.actor.insert_child_at_index(
-                    this.state.dragPlaceholder.actor,
-                    this.state.dragPlaceholderPos
-                );
-            }
-
-            if (fadeIn) this.state.dragPlaceholder.animateIn();
         }
 
+        // get current position
+        let pos = 0;
+        while(pos < this.state.dragging.posList.length && axis[0] > this.state.dragging.posList[pos])
+            pos++;
+
+        // keep pinned and unpinned items separate
+        if((this.state.dragging.isForeign && pos > this.pinnedFavorites._favorites.length) ||
+            (!this.state.dragging.isForeign && source.groupState.isFavoriteApp && pos >= this.pinnedFavorites._favorites.length) ||
+            (!this.state.dragging.isForeign && !source.groupState.isFavoriteApp && pos < this.pinnedFavorites._favorites.length))
+            return DND.DragMotionResult.NO_DROP;
+
+        // handle position change
+        if (pos !== this.state.dragging.pos) {
+            this.state.dragging.pos = pos;
+
+            if(this.state.dragging.isForeign) {
+                if (this.state.dragging.dragPlaceholder)
+                    appList.actor.set_child_at_index(this.state.dragging.dragPlaceholder.actor, pos);
+                else {
+                    let iconSize = this.getPanelIconSize() * global.ui_scale;
+                    this.state.dragging.dragPlaceholder = new DND.GenericDragPlaceholderItem();
+                    this.state.dragging.dragPlaceholder.child.width = iconSize;
+                    this.state.dragging.dragPlaceholder.child.height = iconSize;
+                    appList.actor.insert_child_at_index(
+                        this.state.dragging.dragPlaceholder.actor,
+                        this.state.dragging.pos
+                    );
+                    this.state.dragging.dragPlaceholder.animateIn();
+                }
+            }
+            else
+                appList.actor.set_child_at_index(source.actor, pos);
+        }
+
+        if(this.state.dragging.isForeign)
+            return DND.DragMotionResult.COPY_DROP;
         return DND.DragMotionResult.MOVE_DROP;
     }
 
     // TODO: Figure out exactly which properties on this applet constructor the Cinnamon APIs needs for all modes of
     // DND, so we can kill the _delegate reference. Long term, a PR to Cinnamon should be opened fixing circular
     // object reference structures for the applet and desklet classes.
-    acceptDrop(source, actor, x) {
-        if (!this.state.settings.enableDragging || this.state.panelEditMode) {
+    acceptDrop(source, actor) {
+        if (!this.state.settings.enableDragging || this.state.panelEditMode)
             return false;
-        }
-        if (typeof source.groupState === 'undefined') {
+
+        // add new launcher
+        if (this.state.dragging.isForeign) {
+            let pos = this.state.dragging.pos;
+            this.clearDragPlaceholder();
+            this.clearDragParameters();
+
             let appId = source.isDraggableApp ? source.get_app_id() : source.getId();
             if (appId) {
-                this.acceptNewLauncher(appId);
+                this.acceptNewLauncher(appId, pos);
                 return true;
             }
             return false;
         }
 
-        let currentAppList = this.getCurrentAppList();
-        let pos = this.state.dragPlaceholderPos;
-        this.clearDragPlaceholder();
+        // move existing launcher
+        return this.moveLauncher(source);
+    }
 
-        // For getting the fallback position, possibly dead code
-        if (pos === -1) {
-            let children = currentAppList.actor.get_children();
-            let _pos = 0;
-            for (let i = 0, len = children.length; i < len; i++) {
-                if (x > children[i].get_allocation_box().x1 + children[i].width / 2) {
-                    _pos = i;
-                }
-            }
-            if (_pos !== this.state.dragPlaceholderPos) {
-                pos = _pos;
-            }
+    clearDragParameters() {
+        this.state.dragging.isForeign = null;
+        this.state.dragging.posList = null;
+        this.state.dragging.pos = -1;
+    }
+
+    clearDragPlaceholder(animate = false) {
+        if(this.state.dragging.dragPlaceholder) {
+            if(animate)
+                this.state.dragging.dragPlaceholder.animateOutAndDestroy();
+            else
+                this.state.dragging.dragPlaceholder.actor.destroy();
+            this.state.dragging.dragPlaceholder = null;
         }
+    }
+
+    handleDragOut() {
+        if(this.state.dragging.isForeign) {
+            this.clearDragPlaceholder(true);
+            this.clearDragParameters();
+        }
+    }
+
+    acceptNewLauncher(path, pos = -1) {
+        this.pinnedFavorites.addFavorite({appId: path, pos: pos});
+        // Need to determine why the favorites setting signal doesn't emit outside the applet actions
+        this.updateFavorites();
+    }
+
+    moveLauncher(source) {
+        let pos = this.state.dragging.pos;
+        this.clearDragParameters();
 
         Meta.later_add(Meta.LaterType.BEFORE_REDRAW, () => {
-            // Move the button
-            currentAppList.actor.set_child_at_index(source.actor, pos);
-            currentAppList.updateAppGroupIndexes();
+            this.getCurrentAppList().updateAppGroupIndexes();
             // Refresh the group's thumbnails so hoverMenu is aware of the position change
             // In the case of dragging a group that has a delay before Cinnamon can grab its
             // thumbnail texture, e.g., LibreOffice, defer the refresh.
@@ -796,7 +894,6 @@ class GroupedWindowListApplet extends Applet.Applet {
                 setTimeout(() => source.groupState.trigger('windowCount'), 0);
             }
 
-
             // Handle favoriting if pin on drag is enabled
             if (!source.groupState.app.is_window_backed()) {
                 let opts = {
@@ -816,20 +913,6 @@ class GroupedWindowListApplet extends Applet.Applet {
         return true;
     }
 
-    clearDragPlaceholder() {
-        if (this.state.dragPlaceholder) {
-            this.state.dragPlaceholder.animateOutAndDestroy();
-            this.state.dragPlaceholder = null;
-            this.state.dragPlaceholderPos = -1;
-        }
-    }
-
-    acceptNewLauncher(path) {
-        this.pinnedFavorites.addFavorite({appId: path, pos: -1});
-        // Need to determine why the favorites setting signal doesn't emit outside the applet actions
-        this.updateFavorites();
-    }
-
     onWorkspaceRemoved(metaScreen, index) {
         if (this.appLists.length <= index) {
             return;
@@ -838,14 +921,16 @@ class GroupedWindowListApplet extends Applet.Applet {
         for (let i = 0; i < this.appLists.length; i++) {
             let workspaceIndex = this.appLists[i].metaWorkspace.index();
             if (workspaceIndex === -1) {
-                this.appLists[i].destroy();
-                this.appLists[i] = null;
+                if (this.appLists[i] != null) {
+                    this.appLists[i].destroy();
+                    this.appLists[i] = null;
+                }
                 removedLists.push(i);
             } else {
                 this.appLists[i].index = workspaceIndex;
             }
         }
-        for (let i = 0; i < removedLists.length; i++) {
+        for (let i = removedLists.length - 1; i >= 0; i--) {
             this.appLists.splice(removedLists[i], 1);
         }
         this.state.set({currentWs: global.screen.get_active_workspace_index()});
