@@ -9,13 +9,12 @@ const PopupMenu = imports.ui.popupMenu;
 const Applet = imports.ui.applet;
 const SignalManager = imports.misc.signalManager;
 
-const {each, findIndex, tryFn, unref, trySpawnCommandLine, spawn_async} = imports.misc.util;
+const {each, findIndex, tryFn, unref, trySpawnCommandLine, spawn_async, getDesktopActionIcon} = imports.misc.util;
 const {
     CLOSE_BTN_SIZE,
     CLOSED_BUTTON_STYLE,
     OPACITY_OPAQUE,
     RESERVE_KEYS,
-    ICON_NAMES,
     FavType,
     autoStartStrDir
 } = require('./constants');
@@ -107,7 +106,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
                 }
             }
             // Workspace
-            if ((length = global.screen.n_workspaces) > 1) {
+            if ((length = global.workspace_manager.n_workspaces) > 1) {
                 if (this.groupState.lastFocused && this.groupState.lastFocused.is_on_all_workspaces()) {
                     item = createMenuItem({label: _('Only on this workspace')});
                     this.signals.connect(item, 'activate', () => {
@@ -131,7 +130,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
 
                     let connectWorkspaceEvent = (ws, j) => {
                         this.signals.connect(ws, 'activate', () => {
-                            this.groupState.lastFocused.change_workspace(global.screen.get_workspace_by_index(j));
+                            this.groupState.lastFocused.change_workspace(global.workspace_manager.get_workspace_by_index(j));
                         });
                     };
                     for (let i = 0; i < length; i++) {
@@ -242,21 +241,9 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             if (actions) {
                 this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
                 let handleAction = (action) => {
-                    let actionID = '';
-                    if (action.toUpperCase() === action) {
-                        actionID = action.toLowerCase();
-                    } else {
-                        // first letter lowercase, replace uppercase with _+lowercase
-                        actionID = action.charAt(0).toLowerCase() + action.slice(1);
-                        actionID = actionID.replace(/([A-Z])/g, '_$1').toLowerCase();
-                    }
-                    actionID = actionID.replace(/-/g, '_');
-
-                    let icon = 'application-x-executable';
-
-                    if (ICON_NAMES.hasOwnProperty(actionID)) {
-                        icon = ICON_NAMES[actionID];
-                    }
+                    let icon = getDesktopActionIcon(action);
+                    if (icon == null)
+                        icon = 'application-x-executable';
 
                     item = createMenuItem({
                         label: _(this.groupState.appInfo.get_action_name(action)),
@@ -383,11 +370,15 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
 
     onToggled(actor, isOpening) {
         this.state.set({menuOpen: this.isOpen});
+    }
 
-        if (!isOpening) return;
+    toggle() {
+        if (!this.isOpen) {
+            this.removeAll();
+            this.populateMenu();
+        }
 
-        this.removeAll();
-        this.populateMenu();
+        Applet.AppletPopupMenu.prototype.toggle.call(this);
     }
 
     toggleAutostart() {
@@ -626,7 +617,7 @@ class WindowThumbnail {
         if (actorString.indexOf('this.actor') > -1
             && (!this.lastEnterActor
                 || (this.lastEnterActor.indexOf('StButton') === -1)
-                    && this.lastEnterActor.indexOf('ClutterClone') === -1)) {
+                    && this.lastEnterActor.indexOf('ClutterActor') === -1)) {
             this.destroyOverlayPreview();
             this.hoverPeek(this.state.settings.peekOpacity);
         }
@@ -717,16 +708,17 @@ class WindowThumbnail {
         }
         // Create our own thumbnail if it doesn't exist
         if (this.metaWindowActor) {
-            this.signals.disconnect('size-changed', this.metaWindowActor);
+            this.disconnectSizeNotify();
         } else {
             this.metaWindowActor = this.metaWindow.get_compositor_private();
         }
         if (this.metaWindowActor && !this.metaWindowActor.is_finalized()) {
-            this.signals.connect(this.metaWindowActor, 'size-changed', () => this.refreshThumbnail());
+            this.signals.connect(this.metaWindow, 'unmanaging', () => this.disconnectSizeNotify());
+            this.signals.connect(this.metaWindowActor, 'notify::size', () => this.refreshThumbnail());
 
             let windowTexture = this.metaWindowActor.get_texture();
             if (!windowTexture) return;
-            let [width, height] = windowTexture.get_size();
+            let [width, height] = this.metaWindowActor.get_size();
             let scale = Math.min(1.0, thumbnailWidth / width, thumbnailHeight / height) * global.ui_scale;
             width = Math.round(width * scale);
             height = Math.round(height * scale);
@@ -737,8 +729,8 @@ class WindowThumbnail {
                 this.thumbnailActor.child.width = width;
                 this.thumbnailActor.child.height = height;
             } else {
-                this.thumbnailActor.child = new Clutter.Clone({
-                    source: windowTexture,
+                this.thumbnailActor.child = new Clutter.Actor({
+                    content: windowTexture,
                     reactive: true,
                     width,
                     height
@@ -749,6 +741,11 @@ class WindowThumbnail {
         }
     }
 
+    disconnectSizeNotify(actor) {
+        this.signals.disconnect('unmanaging', this.metaWindow);
+        this.signals.disconnect('notify::size', this.metaWindowActor);
+    }
+
     refreshThumbnail() {
         if (this.willUnmount
             || !this.groupState
@@ -831,13 +828,16 @@ class WindowThumbnail {
             this.metaWindowActor = this.metaWindow.get_compositor_private();
         }
         this.state.set({
-            lastOverlayPreview: new Clutter.Clone({
-                source: this.metaWindowActor.get_texture(),
+            lastOverlayPreview: new Clutter.Actor({
+                content: this.metaWindowActor.get_texture(),
                 opacity: 0
             })
         });
+
         let [x, y] = this.metaWindowActor.get_position();
+        let [width, height] = this.metaWindowActor.get_size();
         this.state.lastOverlayPreview.set_position(x, y);
+        this.state.lastOverlayPreview.set_size(width, height);
         global.overlay_group.add_child(this.state.lastOverlayPreview);
         global.overlay_group.set_child_above_sibling(this.state.lastOverlayPreview, null);
         setOpacity(this.state.settings.peekTimeIn, this.state.lastOverlayPreview, opacity);
