@@ -46,6 +46,8 @@
 
 const Cinnamon = imports.gi.Cinnamon;
 const Clutter = imports.gi.Clutter;
+const Cogl = imports.gi.Cogl;
+const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const Gdk = imports.gi.Gdk;
 const Lang = imports.lang;
@@ -75,7 +77,6 @@ class WindowPreview extends Tooltips.TooltipBase {
         this._applet = item._applet;
         this.metaWindow = metaWindow;
         this._windowActor = null;
-        let x = this.windowActor;
         this.uiScale = global.ui_scale;
         this.thumbScale = previewScale;
 
@@ -154,16 +155,20 @@ class WindowPreview extends Tooltips.TooltipBase {
             return;
         }
 
-        let [width, height] = this._getScaledTextureSize(windowTexture);
+        let [width, height] = this._getScaledTextureSize(this.windowActor);
 
-        this.thumbnail = new Clutter.Clone({
-            source: windowTexture,
+        this.thumbnail = new Clutter.Actor({
+            content: windowTexture,
             width: width,
             height: height
         });
 
-        this._sizeChangedId = this.windowActor.connect('size-changed', () => {
-            let [width, height] = this._getScaledTextureSize(windowTexture);
+        this._sizeChangedId = this.windowActor.connect('notify::size', () => {
+            if (this.thumbnail === null) {
+                return;
+            }
+
+            let [width, height] = this._getScaledTextureSize(this.windowActor);
             this.thumbnail.set_size(width, height);
             this._set_position();
         });
@@ -254,7 +259,7 @@ class WindowPreview extends Tooltips.TooltipBase {
 }
 
 class AppMenuButton {
-    constructor(applet, metaWindow, alert) {
+    constructor(applet, metaWindow, transient) {
         this.actor = new Cinnamon.GenericContainer({
             name: 'appMenu',
             style_class: 'window-list-item-box',
@@ -264,10 +269,14 @@ class AppMenuButton {
 
         this._applet = applet;
         this.metaWindow = metaWindow;
-        this.alert = alert || metaWindow.is_demanding_attention() || metaWindow.is_urgent();
-        this.labelVisible = false;
+        this.transient = transient;
+
+        let initially_urgent = transient || metaWindow.demands_attention || metaWindow.urgent;
+        this.drawLabel = false;
+        this.labelVisiblePref = false;
         this._signals = new SignalManager.SignalManager();
-        this.xid = metaWindow.get_xwindow();
+        this.xid = global.screen.get_xwindow_for_window(metaWindow);
+        this._flashTimer = null;
 
         if (this._applet.orientation == St.Side.TOP)
             this.actor.add_style_class_name('top');
@@ -286,6 +295,11 @@ class AppMenuButton {
                 Lang.bind(this, this._getPreferredWidth));
         this._signals.connect(this.actor, 'get-preferred-height',
                 Lang.bind(this, this._getPreferredHeight));
+
+        this._signals.connect(this.actor, 'notify::allocation',
+                              Lang.bind(this, this.updateIconGeometry));
+        this._updateIconGeometryTimeoutId = 0;
+
         this._signals.connect(this.actor, 'allocate', Lang.bind(this, this._allocate));
 
         this.progressOverlay = new St.Widget({ style_class: "progress", reactive: false, important: true  });
@@ -293,10 +307,6 @@ class AppMenuButton {
         this.actor.add_actor(this.progressOverlay);
 
         this._iconBox = new Cinnamon.Slicer({ name: 'appMenuIcon' });
-        this._signals.connect(this._iconBox, 'style-changed',
-                              Lang.bind(this, this._onIconBoxStyleChanged));
-        this._signals.connect(this._iconBox, 'notify::allocation',
-                              Lang.bind(this, this._updateIconBoxClipAndGeometry));
         this.actor.add_actor(this._iconBox);
 
         this._label = new St.Label();
@@ -304,7 +314,6 @@ class AppMenuButton {
 
         this.updateLabelVisible();
 
-        this._iconBottomClip = 0;
         this._visible = true;
 
         this._progress = 0;
@@ -315,7 +324,7 @@ class AppMenuButton {
                 this.progressOverlay.show();
             } else
                 this.progressOverlay.hide();
-            this._updateProgressId = this.metaWindow.connect("notify::progress", () => {
+            this._updateProgressId = this._signals.connect(this.metaWindow, "notify::progress", () => {
                 if (this.metaWindow.progress != this._progress) {
                     this._progress = this.metaWindow.progress;
 
@@ -336,7 +345,7 @@ class AppMenuButton {
 
         this.onPreviewChanged();
 
-        if (!this.alert) {
+        if (!this.transient) {
             this._menuManager = new PopupMenu.PopupMenuManager(this);
             this.rightClickMenu = new AppMenuButtonRightClickMenu(this, this.metaWindow, this._applet.orientation);
             this._menuManager.addMenu(this.rightClickMenu);
@@ -360,13 +369,13 @@ class AppMenuButton {
         this.onFocus();
         this.setIcon();
 
-        if (this.alert)
+        if (initially_urgent)
             this.getAttention();
 
         this._signals.connect(this.metaWindow, 'notify::title', this.setDisplayTitle, this);
         this._signals.connect(this.metaWindow, "notify::minimized", this.setDisplayTitle, this);
         this._signals.connect(this.metaWindow, "notify::tile-type", this.setDisplayTitle, this);
-        this._signals.connect(this.metaWindow, "icon-changed", this.setIcon, this);
+        this._signals.connect(this.metaWindow, "notify::icon", this.setIcon, this);
         this._signals.connect(this.metaWindow, "notify::appears-focused", this.onFocus, this);
         this._signals.connect(this.metaWindow, "unmanaged", this.onUnmanaged, this);
     }
@@ -516,20 +525,26 @@ class AppMenuButton {
 
         if (this.metaWindow.minimized) {
             title = "["+ title +"]";
-        } else if (this.metaWindow.tile_type == Meta.WindowTileType.TILED) {
+        } else if (this.metaWindow.tile_mode != Meta.TileMode.NONE && this.metaWindow.tile_mode != Meta.TileMode.MAXIMIZED) {
             title = "|"+ title;
         }
-        else if (this.metaWindow.tile_type == Meta.WindowTileType.SNAPPED) {
-            title = "||"+ title;
-        }
 
         this._label.set_text(title);
     }
 
     destroy() {
+        if (this._flashTimer) {
+            Mainloop.source_remove(this._flashTimer);
+            this._flashTimer = null;
+        }
+        if (this._updateIconGeometryTimeoutId > 0) {
+            Mainloop.source_remove(this._updateIconGeometryTimeoutId);
+            this._updateIconGeometryTimeoutId = 0;
+        }
+
         this._signals.disconnectAllSignals();
         this._tooltip.destroy();
-        if (!this.alert) {
+        if (!this.transient) {
             this.rightClickMenu.destroy();
             this._menuManager.destroy();
         }
@@ -561,7 +576,7 @@ class AppMenuButton {
             this.actor.remove_style_class_name("window-list-item-demands-attention-top");
             this._needsAttention = false;
 
-            if (this.alert) {
+            if (this.transient) {
                 this.destroy();
                 this._windows.splice(this._windows.indexOf(this), 1);
             }
@@ -572,7 +587,7 @@ class AppMenuButton {
 
     _onButtonRelease(actor, event) {
         this._tooltip.hide();
-        if (this.alert) {
+        if (this.transient) {
             if (event.get_button() == 1)
                 this._toggleWindow(false);
             return false;
@@ -591,7 +606,7 @@ class AppMenuButton {
 
     _onButtonPress(actor, event) {
         this._tooltip.hide();
-        if (!this.alert && event.get_button() == 3) {
+        if (!this.transient && event.get_button() == 3) {
             this.rightClickMenu.mouseEvent = event;
             this.rightClickMenu.toggle();
 
@@ -611,42 +626,44 @@ class AppMenuButton {
         }
     }
 
-    _onIconBoxStyleChanged() {
-        let node = this._iconBox.get_theme_node();
-        this._iconBottomClip = node.get_length('app-icon-bottom-clip');
-        this._updateIconBoxClipAndGeometry();
+    updateIconGeometry() {
+        if (this._updateIconGeometryTimeoutId > 0) {
+            Mainloop.source_remove(this._updateIconGeometryTimeoutId);
+        }
+
+        this._updateIconGeometryTimeoutId = Mainloop.timeout_add(50, Lang.bind(this, this._updateIconGeometryTimeout));
     }
 
-    _updateIconBoxClipAndGeometry() {
-        let allocation = this._iconBox.allocation;
-        if (this._iconBottomClip > 0)
-            this._iconBox.set_clip(0, 0,
-                   allocation.x2 - allocation.x1,
-                   allocation.y2 - allocation.y1 - this._iconBottomClip);
-        else
-            this._iconBox.remove_clip();
+    _updateIconGeometryTimeout() {
+        this._updateIconGeometryTimeoutId = 0;
 
         let rect = new Meta.Rectangle();
         [rect.x, rect.y] = this.actor.get_transformed_position();
         [rect.width, rect.height] = this.actor.get_transformed_size();
 
         this.metaWindow.set_icon_geometry(rect);
+
+        return GLib.SOURCE_REMOVE;
     }
 
     _getPreferredWidth(actor, forHeight, alloc) {
         let [minSize, naturalSize] = this._iconBox.get_preferred_width(forHeight);
-        // minimum size just enough for icon if we ever get that many apps going
-        alloc.min_size = naturalSize;
+
+        alloc.min_size = 1 * global.ui_scale;
 
         if (this._applet.orientation == St.Side.TOP || this._applet.orientation == St.Side.BOTTOM ) {
-        // the 'buttons use entire space' option only makes sense on horizontal panels
-            if (this._applet.buttonsUseEntireSpace) {
-                let [lminSize, lnaturalSize] = this._label.get_preferred_width(forHeight);
-                let spacing = this.actor.get_theme_node().get_length('spacing');
-                alloc.natural_size = Math.max(150 * global.ui_scale,
-                        naturalSize + spacing + lnaturalSize);
+        // the 'buttons use entire space' option only makes sense on horizontal panels with labels
+            if (this.labelVisiblePref) {
+                if (this._applet.buttonsUseEntireSpace) {
+                    let [lminSize, lnaturalSize] = this._label.get_preferred_width(forHeight);
+                    let spacing = this.actor.get_theme_node().get_length('spacing');
+                    alloc.natural_size = Math.max(150 * global.ui_scale,
+                            naturalSize + spacing + lnaturalSize);
+                } else {
+                    alloc.natural_size = this._applet.buttonWidth * global.ui_scale;
+                }
             } else {
-                alloc.natural_size = 150 * global.ui_scale;
+                alloc.natural_size = naturalSize
             }
         } else {
             alloc.natural_size = this._applet._panelHeight;
@@ -656,7 +673,7 @@ class AppMenuButton {
     _getPreferredHeight(actor, forWidth, alloc) {
         let [minSize1, naturalSize1] = this._iconBox.get_preferred_height(forWidth);
 
-        if (this.labelVisible) {
+        if (this.labelVisiblePref) {
             let [minSize2, naturalSize2] = this._label.get_preferred_height(forWidth);
             alloc.min_size = Math.max(minSize1, minSize2);
         } else {
@@ -691,7 +708,13 @@ class AppMenuButton {
         childBox.y1 = box.y1 + yPadding;
         childBox.y2 = childBox.y1 + Math.min(naturalHeight, allocHeight);
 
-        if (this.labelVisible) {
+        if (allocWidth < naturalWidth) {
+            this.labelVisible = false;
+        } else {
+            this.labelVisible = this.labelVisiblePref;
+        }
+
+        if (this.drawLabel) {
             if (direction === Clutter.TextDirection.LTR) {
                 childBox.x1 = box.x1;
             } else {
@@ -704,7 +727,7 @@ class AppMenuButton {
         }
         this._iconBox.allocate(childBox, flags);
 
-        if (this.labelVisible) {
+        if (this.drawLabel) {
             [minWidth, minHeight, naturalWidth, naturalHeight] = this._label.get_preferred_size();
 
             yPadding = Math.floor(Math.max(0, allocHeight - naturalHeight) / 2);
@@ -738,12 +761,14 @@ class AppMenuButton {
     }
 
     updateLabelVisible() {
-        if (this._applet.orientation == St.Side.TOP || this._applet.orientation == St.Side.BOTTOM) {
+        if (this._applet.showLabelPanel) {
             this._label.show();
-            this.labelVisible = true;
+            this.labelVisiblePref = true;
+            this.drawLabel = true;
         } else {
             this._label.hide();
-            this.labelVisible = false;
+            this.labelVisiblePref = false;
+            this.drawLabel = false;
         }
     }
 
@@ -776,22 +801,28 @@ class AppMenuButton {
     }
 
     _flashButton() {
-        if (!this._needsAttention)
+        if (!this._needsAttention || this._flashTimer)
             return;
 
         let counter = 0;
-        let sc = "window-list-item-demands-attention";
+        const sc = "window-list-item-demands-attention";
 
-        Mainloop.timeout_add(FLASH_INTERVAL, () => {
-            if (!this._needsAttention)
+        this._flashTimer = Mainloop.timeout_add(FLASH_INTERVAL, () => {
+            if (!this._needsAttention) {
+                this._flashTimer = null;
                 return false;
+            }
 
             if (this.actor.has_style_class_name(sc))
                 this.actor.remove_style_class_name(sc);
             else
                 this.actor.add_style_class_name(sc);
 
-            return counter++ < FLASH_MAX_COUNT;
+            const continueFlashing = (counter++ < FLASH_MAX_COUNT);
+            if (!continueFlashing) {
+                this._flashTimer = null;
+            }
+            return continueFlashing;
         });
     }
 };
@@ -836,7 +867,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
         }
 
         // Move to workspace
-        if ((length = global.screen.n_workspaces) > 1) {
+        if ((length = global.workspace_manager.n_workspaces) > 1) {
             if (mw.is_on_all_workspaces()) {
                 item = new PopupMenu.PopupMenuItem(_("Only on this workspace"));
                 this._signals.connect(item, 'activate', function() {
@@ -864,7 +895,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
                         ws.setSensitive(false);
 
                     this._signals.connect(ws, 'activate', function() {
-                        mw.change_workspace(global.screen.get_workspace_by_index(j));
+                        mw.change_workspace(global.workspace_manager.get_workspace_by_index(j));
                     });
                     item.menu.addMenuItem(ws);
                 }
@@ -1016,17 +1047,19 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.settings.bind("reverse-scrolling", "reverseScroll");
         this.settings.bind("left-click-minimize", "leftClickMinimize");
         this.settings.bind("middle-click-close", "middleClickClose");
+        this.settings.bind("button-width", "buttonWidth", this._refreshAllItems);
         this.settings.bind("buttons-use-entire-space", "buttonsUseEntireSpace", this._refreshAllItems);
+        this.settings.bind("panel-show-label", "showLabelPanel", this._updateLabels);
         this.settings.bind("window-preview", "usePreview", this._onPreviewChanged);
         this.settings.bind("window-preview-show-label", "showLabel", this._onPreviewChanged);
         this.settings.bind("window-preview-scale", "previewScale", this._onPreviewChanged);
         this.settings.bind("last-window-order", "lastWindowOrder", null);
 
-        this.signals.connect(global.screen, 'window-added', this._onWindowAddedAsync, this);
+        this.signals.connect(global.display, 'window-created', this._onWindowAddedAsync, this);
         this.signals.connect(global.screen, 'window-monitor-changed', this._onWindowMonitorChanged, this);
         this.signals.connect(global.screen, 'window-workspace-changed', this._onWindowWorkspaceChanged, this);
         this.signals.connect(global.screen, 'window-skip-taskbar-changed', this._onWindowSkipTaskbarChanged, this);
-        this.signals.connect(global.screen, 'monitors-changed', this._updateWatchedMonitors, this);
+        this.signals.connect(Main.panelManager, 'monitors-changed', this._updateWatchedMonitors, this);
         this.signals.connect(global.window_manager, 'switch-workspace', this._refreshAllItems, this);
         this.signals.connect(Cinnamon.WindowTracker.get_default(), "window-app-changed", this._onWindowAppChanged, this);
 
@@ -1113,6 +1146,8 @@ class CinnamonWindowListApplet extends Applet.Applet {
         if (this.appletEnabled) {
             this._updateSpacing();
         }
+
+        this._updateAllIconGeometry()
     }
 
     _updateSpacing() {
@@ -1194,13 +1229,13 @@ class CinnamonWindowListApplet extends Applet.Applet {
         if (!this._windows[i].getAttention())
             return;
 
-        if (window.get_workspace() != global.screen.get_active_workspace())
+        if (window.get_workspace() != global.workspace_manager.get_active_workspace())
             this._addWindow(window, true);
     }
 
     _refreshItem(window) {
         window.actor.visible =
-            (window.metaWindow.get_workspace() == global.screen.get_active_workspace()) ||
+            (window.metaWindow.get_workspace() == global.workspace_manager.get_active_workspace()) ||
             window.metaWindow.is_on_all_workspaces() ||
             this.showAllWorkspaces;
 
@@ -1208,11 +1243,13 @@ class CinnamonWindowListApplet extends Applet.Applet {
          * AppMenuButton. If this is actually a temporary AppMenuButton for
          * urgent windows on other workspaces, it is shown iff the normal
          * one isn't shown! */
-        if (window.alert)
+        if (window.transient)
             window.actor.visible = !window.actor.visible;
 
         if (window.actor.visible)
             window.setIcon();
+
+        window.updateIconGeometry();
     }
 
     _refreshAllItems() {
@@ -1226,9 +1263,21 @@ class CinnamonWindowListApplet extends Applet.Applet {
             window.setDisplayTitle();
         }
     }
+    
+    _updateLabels() {
+        for (let window of this._windows)
+            window.updateLabelVisible();
+    }
 
     _updateWatchedMonitors() {
-        let n_mons = Gdk.Screen.get_default().get_n_monitors();
+        // this can be called after our settings are finalized (and those attributes deleted),
+        // so lastWindowOrder won't exist anymore. This can happen when panels are removed, for
+        // example due to monitor changes.
+        if (this.lastWindowOrder === undefined) {
+            return;
+        }
+
+        let n_mons = global.display.get_n_monitors();
         let on_primary = this.panel.monitorIndex == Main.layoutManager.primaryIndex;
         let instances = Main.AppletManager.getRunningInstancesForUuid(this._uuid);
 
@@ -1257,8 +1306,8 @@ class CinnamonWindowListApplet extends Applet.Applet {
         // Now track the windows in our favorite monitors
         let windows = global.display.list_windows(0);
         if (this.showAllWorkspaces) {
-            for (let wks=0; wks<global.screen.n_workspaces; wks++) {
-                let metaWorkspace = global.screen.get_workspace_by_index(wks);
+            for (let wks=0; wks<global.workspace_manager.n_workspaces; wks++) {
+                let metaWorkspace = global.workspace_manager.get_workspace_by_index(wks);
                 let wks_windows = metaWorkspace.list_windows();
                 for (let wks_window of wks_windows) {
                     windows.push(wks_window);
@@ -1278,15 +1327,16 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.refreshing = false;
 
         this._applySavedOrder();
+        this._updateAllIconGeometry();
     }
 
-    _addWindow(metaWindow, alert) {
+    _addWindow(metaWindow, transient) {
         for (let window of this._windows)
             if (window.metaWindow == metaWindow &&
-                window.alert == alert)
+                window.transient == transient)
                 return;
 
-        let appButton = new AppMenuButton(this, metaWindow, alert);
+        let appButton = new AppMenuButton(this, metaWindow, transient);
         this.manager_container.add_actor(appButton.actor);
 
         this._windows.push(appButton);
@@ -1295,11 +1345,11 @@ class CinnamonWindowListApplet extends Applet.Applet {
          * workspace. So if we add an AppMenuButton for a window in another
          * workspace, put it in the right position. It is at the end by
          * default, so move it to the start if needed */
-        if (alert) {
-            if (metaWindow.get_workspace().index() < global.screen.get_active_workspace_index())
+        if (transient) {
+            if (metaWindow.get_workspace().index() < global.workspace_manager.get_active_workspace_index())
                 this.manager_container.set_child_at_index(appButton.actor, 0);
         } else {
-            if (metaWindow.get_workspace() != global.screen.get_active_workspace()) {
+            if (metaWindow.get_workspace() != global.workspace_manager.get_active_workspace()) {
                 if (!(this.showAllWorkspaces)) {
                     appButton.actor.hide();
                 }
@@ -1307,6 +1357,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
         }
 
         this._saveOrder();
+        this._updateAllIconGeometry();
     }
 
     _removeWindow(metaWindow) {
@@ -1320,6 +1371,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
         }
 
         this._saveOrder();
+        this._updateAllIconGeometry();
     }
 
     _shouldAdd(metaWindow) {
@@ -1374,6 +1426,12 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.lastWindowOrder = new_order.join("::");
     }
 
+    _updateAllIconGeometry() {
+        for (let window of this._windows) {
+            window.updateIconGeometry();
+        }
+    }
+
     handleDragOver(source, actor, x, y, time) {
         if (this._inEditMode)
             return DND.DragMotionResult.MOVE_DROP;
@@ -1381,7 +1439,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
             return DND.DragMotionResult.NO_DROP;
 
         let children = this.manager_container.get_children();
-        let isVertical = this.manager_container.height > this.manager_container.width;
+        let isVertical = (this.orientation == St.Side.LEFT || this.orientation == St.Side.RIGHT);
         let axis = isVertical ? [y, 'y1'] : [x, 'x1'];
 
         this._dragPlaceholderPos = -1;
@@ -1389,7 +1447,8 @@ class CinnamonWindowListApplet extends Applet.Applet {
         for(let i = 0; i < children.length; i++) {
             if (!children[i].visible)
                 continue;
-            let dist = Math.abs(axis[0] - (children[i].get_allocation_box()[axis[1]] + children[i].width / 2));
+            let dim = isVertical ? children[i].height : children[i].width;
+            let dist = Math.abs(axis[0] - (children[i].get_allocation_box()[axis[1]] + dim / 2));
             if(dist < minDist || minDist == -1) {
                 minDist = dist;
                 this._dragPlaceholderPos = i;
@@ -1419,6 +1478,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.manager_container.set_child_at_index(source.actor, this._dragPlaceholderPos);
 
         this._saveOrder();
+        this._updateAllIconGeometry();
 
         return true;
     }
