@@ -3,20 +3,19 @@
 import os
 import subprocess
 
-import dbus
 import gi
 gi.require_version('Gtk', '3.0')
-gi.require_version('CDesktopEnums', '3.0')
-from gi.repository import Gio, Gtk, GObject, GLib, XApp
+from gi.repository import Gio, Gtk, GObject, GLib
 
 from xapp.SettingsWidgets import SettingsWidget, SettingsLabel
 from xapp.GSettingsWidgets import PXGSettingsBackend
-from ChooserButtonWidgets import DateChooserButton, TweenChooserButton, EffectChooserButton
+from ChooserButtonWidgets import DateChooserButton, TimeChooserButton
 from KeybindingWidgets import ButtonKeybinding
+import util
 
 settings_objects = {}
 
-CAN_BACKEND = ["SoundFileChooser", "TweenChooser", "EffectChooser", "DateChooser", "Keybinding"]
+CAN_BACKEND = ["SoundFileChooser", "DateChooser", "TimeChooser", "Keybinding"]
 
 class BinFileMonitor(GObject.GObject):
     __gsignals__ = {
@@ -360,29 +359,13 @@ class SoundFileChooser(SettingsWidget):
         self.play_button.connect("clicked", self.on_play_clicked)
         self.content_widget.pack_start(self.play_button, False, False, 0)
 
-        self._proxy = None
-
-        try:
-            Gio.DBusProxy.new_for_bus(Gio.BusType.SESSION, Gio.DBusProxyFlags.NONE, None,
-                                      'org.cinnamon.SettingsDaemon.Sound',
-                                      '/org/cinnamon/SettingsDaemon/Sound',
-                                      'org.cinnamon.SettingsDaemon.Sound',
-                                      None, self._on_proxy_ready, None)
-        except dbus.exceptions.DBusException as e:
-            print(e)
-            self._proxy = None
-            self.play_button.set_sensitive(False)
-
         self.set_tooltip_text(tooltip)
 
         if size_group:
             self.add_to_size_group(size_group)
 
-    def _on_proxy_ready (self, object, result, data=None):
-        self._proxy = Gio.DBusProxy.new_for_bus_finish(result)
-
     def on_play_clicked(self, widget):
-        self._proxy.PlaySoundFile("(us)", 0, self.get_value())
+        util.play_sound_file(self.get_value())
 
     def on_picker_clicked(self, widget):
         dialog = Gtk.FileChooserDialog(title=self.label.get_text(),
@@ -423,44 +406,6 @@ class SoundFileChooser(SettingsWidget):
     def connect_widget_handlers(self, *args):
         pass
 
-class TweenChooser(SettingsWidget):
-    bind_prop = "tween"
-    bind_dir = Gio.SettingsBindFlags.DEFAULT
-
-    def __init__(self, label, size_group=None, dep_key=None, tooltip=""):
-        super(TweenChooser, self).__init__(dep_key=dep_key)
-
-        self.label = SettingsLabel(label)
-
-        self.content_widget = TweenChooserButton()
-
-        self.pack_start(self.label, False, False, 0)
-        self.pack_end(self.content_widget, False, False, 0)
-
-        self.set_tooltip_text(tooltip)
-
-        if size_group:
-            self.add_to_size_group(size_group)
-
-class EffectChooser(SettingsWidget):
-    bind_prop = "effect"
-    bind_dir = Gio.SettingsBindFlags.DEFAULT
-
-    def __init__(self, label, possible=None, size_group=None, dep_key=None, tooltip=""):
-        super(EffectChooser, self).__init__(dep_key=dep_key)
-
-        self.label = SettingsLabel(label)
-
-        self.content_widget = EffectChooserButton(possible)
-
-        self.pack_start(self.label, False, False, 0)
-        self.pack_end(self.content_widget, False, False, 0)
-
-        self.set_tooltip_text(tooltip)
-
-        if size_group:
-            self.add_to_size_group(size_group)
-
 class DateChooser(SettingsWidget):
     bind_dir = None
 
