@@ -10,19 +10,13 @@ try:
     import html
     import subprocess
     import threading
-    import time
-    import dbus
     from PIL import Image
     import datetime
-    import proxygsettings
     import time
 except Exception as detail:
     print(detail)
     sys.exit(1)
 
-from http.client import HTTPSConnection
-from urllib.parse import urlparse
-
 try:
     import json
 except ImportError:
@@ -163,7 +157,6 @@ class Spice_Harvester(GObject.Object):
         else:
             self.settings = Gio.Settings.new('org.cinnamon')
             self.enabled_key = 'enabled-%ss' % self.collection_type
-        self.settings.connect('changed::%s' % self.enabled_key, self._update_status)
 
         if self.themes:
             self.install_folder = '%s/.themes/' % (home)
@@ -171,6 +164,9 @@ class Spice_Harvester(GObject.Object):
         else:
             self.install_folder = '%s/.local/share/cinnamon/%ss/' % (home, self.collection_type)
             self.spices_directories = ('/usr/share/cinnamon/%ss/' % self.collection_type, self.install_folder)
+            self.settings.connect('changed::%s' % self.enabled_key, self._update_status)
+
+        self._update_status()
 
         self._load_metadata()
 
@@ -191,8 +187,8 @@ class Spice_Harvester(GObject.Object):
         try:
             Gio.DBusProxy.new_for_bus(Gio.BusType.SESSION, Gio.DBusProxyFlags.NONE, None,
                                       'org.Cinnamon', '/org/Cinnamon', 'org.Cinnamon', None, self._on_proxy_ready, None)
-        except dbus.exceptions.DBusException as e:
-            print(e)
+        except GLib.Error as e:
+            print(e.message)
 
     def _on_proxy_ready (self, object, result, data=None):
         try:
@@ -215,6 +211,9 @@ class Spice_Harvester(GObject.Object):
                 self.send_deferred_proxy_calls()
                 return
 
+        if signal_name == "XletsLoadedComplete":
+            self._update_status()
+
         for name, callback in self._proxy_signals:
             if signal_name == name:
                 callback(*params)
@@ -381,28 +380,22 @@ class Spice_Harvester(GObject.Object):
         #Like the one in urllib. Unlike urllib.retrieve url_retrieve
         #can be interrupted. KeyboardInterrupt exception is raised when
         #interrupted.
+        import proxygsettings
+        import requests
+
         count = 0
         blockSize = 1024 * 8
-        parsed_url = urlparse(url)
-        host = parsed_url.netloc
+        proxy_info = proxygsettings.get_proxy_settings()
+
         try:
-            proxy = proxygsettings.get_proxy_settings()
-            if proxy and proxy.get('https'):
-                connection = HTTPSConnection(proxy.get('https'), timeout=15)
-                connection.set_tunnel(host)
-            else:
-                connection = HTTPSConnection(host, timeout=15)
-            headers = { "Accept-Encoding": "identity", "Host": host, "User-Agent": "Python/3" }
-            connection.request("GET", parsed_url.path, headers=headers)
-            urlobj = connection.getresponse()
-            assert urlobj.getcode() == 200
+            response = requests.get(url, proxies=proxy_info, stream=True, timeout=15)
+            assert response.ok
 
-            totalSize = int(urlobj.info()['content-length'])
+            totalSize = int(response.headers.get('content-length'))
 
-            while not self._is_aborted():
-                data = urlobj.read(blockSize)
+            for data in response.iter_content(chunk_size=blockSize):
                 count += 1
-                if not data:
+                if self._is_aborted():
                     break
                 if not binary:
                     data = data.decode("utf-8")
@@ -580,13 +573,9 @@ class Spice_Harvester(GObject.Object):
         self.download_total_files = 0
         self.download_current_file = 0
         self.is_downloading_image_cache = False
-        self.settings.set_int('%s-cache-updated' % self.collection_type, time.time())
         self._advance_queue()
         self.emit('cache-loaded')
 
-    def get_cache_age(self):
-        return (time.time() - self.settings.get_int('%s-cache-updated' % self.collection_type)) / 86400
-
     # checks for corrupt images in the cache so we can redownload them the next time we refresh
     def _is_bad_image(self, path):
         try:
