@@ -13,10 +13,14 @@ const MessageTray = imports.ui.messageTray;
 const Params = imports.misc.params;
 const Mainloop = imports.mainloop;
 
+// don't automatically clear these apps' notifications on window focus
+// lowercase only
+const AUTOCLEAR_BLACKLIST = ['chromium', 'firefox', 'google chrome'];
+
 let nextNotificationId = 1;
 
 // Should really be defined in Gio.js
-const BusIface = 
+const BusIface =
     '<node> \
         <interface name="org.freedesktop.DBus"> \
             <method name="GetConnectionUnixProcessID"> \
@@ -121,6 +125,11 @@ NotificationDaemon.prototype = {
         }
         setting(this, this.settings, "boolean", "removeOld", "remove-old");
         setting(this, this.settings, "int", "timeout", "timeout");
+
+        Cinnamon.WindowTracker.get_default().connect('notify::focus-app',
+            Lang.bind(this, this._onFocusAppChanged));
+        Main.overview.connect('hidden',
+            Lang.bind(this, this._onFocusAppChanged));
     },
 
    // Create an icon for a notification from icon string/path.
@@ -141,25 +150,41 @@ NotificationDaemon.prototype = {
             else if (icon[0] == '/') {
                 let uri = GLib.filename_to_uri(icon, null);
                 return textureCache.load_uri_async(uri, size, size);
-            } else
+            } else {
+                let icon_type = St.IconType.FULLCOLOR;
+                if (icon.search("-symbolic") != -1)
+                    icon_type = St.IconType.SYMBOLIC;
                 return new St.Icon({ icon_name: icon,
-                                     icon_type: St.IconType.FULLCOLOR,
+                                     icon_type: icon_type,
                                      icon_size: size });
+            }
         } else if (hints['image-data']) {
             let [width, height, rowStride, hasAlpha,
                  bitsPerSample, nChannels, data] = hints['image-data'];
             return textureCache.load_from_raw(data, hasAlpha, width, height, rowStride, size);
         } else if (hints['image-path']) {
-            return textureCache.load_uri_async(GLib.filename_to_uri(hints['image-path'], null), size, size);
+            let path = hints['image-path'];
+            if (GLib.path_is_absolute (path)) {
+                return textureCache.load_uri_async(GLib.filename_to_uri(path, null), size, size);
+            } else {
+                let icon_type = St.IconType.FULLCOLOR;
+                if (path.search("-symbolic") != -1) {
+                    icon_type = St.IconType.SYMBOLIC;
+                }
+
+                return new St.Icon({ icon_name: path,
+                                     icon_type: icon_type,
+                                     icon_size: size });
+            }
         } else {
             let stockIcon;
             switch (hints.urgency) {
                 case Urgency.LOW:
                 case Urgency.NORMAL:
-                    stockIcon = 'gtk-dialog-info';
+                    stockIcon = 'dialog-information';
                     break;
                 case Urgency.CRITICAL:
-                    stockIcon = 'gtk-dialog-error';
+                    stockIcon = 'dialog-error';
                     break;
             }
             return new St.Icon({ icon_name: stockIcon,
@@ -211,7 +236,6 @@ NotificationDaemon.prototype = {
         if (!isForTransientNotification) {
             let source = this._lookupSource(title, pid, trayIcon);
             if (source) {
-                source.setTitle(title);
                 return source;
             }
         }
@@ -259,7 +283,7 @@ NotificationDaemon.prototype = {
         this._expireTimer = 0;
         return false;
     },
- 
+
     // Sends a notification to the notification daemon. Returns the id allocated to the notification.
     NotifyAsync: function(params, invocation) {
         let [appName, replacesId, icon, summary, body, actions, hints, timeout] = params;
@@ -295,6 +319,8 @@ NotificationDaemon.prototype = {
                 // early versions of the spec; 'icon_data' should only be used if 'image-path' is not available
                 hints['image-data'] = hints['icon_data'];
 
+        hints['suppress-sound'] = hints.maybeGet('suppress-sound') == true;
+
         let ndata = { appName: appName,
                       icon: icon,
                       summary: summary,
@@ -321,7 +347,7 @@ NotificationDaemon.prototype = {
         } else {    // Custom expiration.
              expires = ndata.expires = Date.now()+timeout;
         }
- 
+
         // Does this notification expire?
         if (expires != 0) {
             // Find place in the notification queue.
@@ -397,7 +423,8 @@ NotificationDaemon.prototype = {
         if (notification == null) {    // Create a new notification!
             notification = new MessageTray.Notification(source, summary, body,
                                                         { icon: iconActor,
-                                                          bannerMarkup: true });
+                                                          bodyMarkup: true,
+                                                          silent: hints['suppress-sound'] });
             ndata.notification = notification;
             notification.connect('destroy', Lang.bind(this,
                 function(n, reason) {
@@ -433,8 +460,8 @@ NotificationDaemon.prototype = {
                 }));
         } else {
             notification.update(summary, body, { icon: iconActor,
-                                                 bannerMarkup: true,
-                                                 clear: true });
+                                                 bodyMarkup: true,
+                                                 silent: hints['suppress-sound'] });
         }
 
         // We only display a large image if an icon is also specified.
@@ -455,6 +482,8 @@ NotificationDaemon.prototype = {
             notification.unsetImage();
         }
 
+        notification.clearButtons();
+
         if (actions.length) {
             notification.setUseActionIcons(hints.maybeGet('action-icons') == true);
             for (let i = 0; i < actions.length - 1; i += 2) {
@@ -507,7 +536,7 @@ NotificationDaemon.prototype = {
             // 'icon-multi',
             'icon-static',
             'persistence',
-            // 'sound',
+            'sound',
         ];
     },
 
@@ -520,6 +549,27 @@ NotificationDaemon.prototype = {
         ];
     },
 
+    _onFocusAppChanged: function() {
+        if (!this._sources.length)
+            return;
+
+        let tracker = Cinnamon.WindowTracker.get_default();
+        if (!tracker.focus_app)
+            return;
+
+        let name = tracker.focus_app.get_name();
+        if (name && AUTOCLEAR_BLACKLIST.includes(name.toLowerCase()))
+            return;
+
+        for (let i = 0; i < this._sources.length; i++) {
+            let source = this._sources[i];
+            if (source.app == tracker.focus_app) {
+                source.destroyNonResidentNotifications();
+                return;
+            }
+        }
+    },
+
     _emitNotificationClosed: function(id, reason) {
         this._dbusImpl.emit_signal('NotificationClosed',
                                    GLib.Variant.new('(uu)', [id, reason]));
@@ -594,37 +644,6 @@ Source.prototype = {
         this.notify(notification);
     },
 
-    handleSummaryClick: function() {
-        if (!this.trayIcon)
-            return false;
-
-        let event = Clutter.get_current_event();
-        if (event.type() != Clutter.EventType.BUTTON_RELEASE)
-            return false;
-
-        // Left clicks are passed through only where there aren't unacknowledged
-        // notifications, so it possible to open them in summary mode; right
-        // clicks are always forwarded, as the right click menu is not useful for
-        // tray icons
-        if (event.get_button() == 1 &&
-            this.notifications.length > 0)
-            return false;
-
-        if (Main.overview.visible) {
-            // We can't just connect to Main.overview's 'hidden' signal,
-            // because it's emitted *before* it calls popModal()...
-            let id = global.connect('notify::stage-input-mode', Lang.bind(this,
-                function () {
-                    global.disconnect(id);
-                    this.trayIcon.click(event);
-                }));
-            Main.overview.hide();
-        } else {
-            this.trayIcon.click(event);
-        }
-        return true;
-    },
-
     _getApp: function() {
         let app;
 
@@ -653,14 +672,13 @@ Source.prototype = {
         // notification-based icons (ie, not a trayicon) or if it was unset before
         if (!this.trayIcon) {
             this.useNotificationIcon = false;
-            
-            let icon = null;                
+            let icon = null;
             if (this.app.get_app_info() != null && this.app.get_app_info().get_icon() != null) {
                 icon = new St.Icon({gicon: this.app.get_app_info().get_icon(), icon_size: this.ICON_SIZE, icon_type: St.IconType.FULLCOLOR});
             }
             if (icon == null) {
-                icon = new St.Icon({icon_name: "application-x-executable", icon_size: this.ICON_SIZE, icon_type: St.IconType.FULLCOLOR});        
-            }            
+                icon = new St.Icon({icon_name: "application-x-executable", icon_size: this.ICON_SIZE, icon_type: St.IconType.FULLCOLOR});
+            }
 
             this._setSummaryIcon(icon);
         }
